<?php

declare(strict_types=1);

namespace mirzaev\vk\arangodb\vk;

// Модуль ArangoDB для фреймворка ВКонтакте
use mirzaev\vk\arangodb\longpoll,
    mirzaev\vk\arangodb\journal;

// Фреймворк ArangoDB
use mirzaev\arangodb\collection,
    mirzaev\arangodb\document;

// Библиотека для работы с API ArabgoDB
use ArangoDBClient\Document as _document,
    ArangoDBClient\Connection as _connection;

/**
 * Чат ВКонтакте
 *
 * @package mirzaev\vk\arangodb\vk
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class chat
{
    /**
     * Инициализация
     *
     * @param _connection $session Сессия соединения с базой данных
     * @param int $id Идентификатор ВКонтакте
     * @param bool $journal Записывать в журнал?
     * @param bool $create Создавать коллекции при их отсутствии?
     *
     * @return ?_document Инстанция документа
     */
    public static function init(_connection $session, int $id, bool $journal = true, bool $create = true): ?_document
    {
        if ($create) {
            // Запрошено создание коллекций в случае их отсутствия

            // Создание коллекции
            collection::init($session, longpoll::COLLECTION_CHATS);
        }

        // Поиск
        if ($chat = static::search($session, $id));
        else if ($chat = document::write($session, longpoll::COLLECTION_CHATS, ['id' => $id]) and $journal)
            journal::init($session, $chat)->write('create', [
                'changes' => [
                    'new' => collection::search($session, sprintf(
                        <<<'AQL'
                            FOR x IN %s
                            FILTER x._id == "%s"
                            LIMIT 1
                            RETURN x
                        AQL,
                        longpoll::COLLECTION_CHATS,
                        $chat
                    )),
                    'old' => null
                ]
            ]);

        // Поиск
        return static::search($session, $id);
    }

    /**
     * Найти в базе данных
     *
     * @param _connection $session Сессия соединения с базой данных
     * @param int $id Идентификатор ВКонтакте
     *
     * @return ?_document Инстанция документа
     */
    public static function search(_connection $session, int $id): ?_document
    {
        return collection::search($session, sprintf(
            <<<'AQL'
                FOR x IN %s
                FILTER x.id == %u
                LIMIT 1
                RETURN x
            AQL,
            longpoll::COLLECTION_CHATS,
            $id
        ));
    }
}
